using System;
using System.Collections.Generic;
using System.IO;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using _Scripts.Utils;
using UnityEngine;

namespace _Scripts.VRBody
{
    public class BoneDataSender : MonoBehaviour
    {
        public enum XsensBones
        {
            Pelvis,
            L5,
            L3,
            T12,
            T8,
            Neck,
            Head,
            RightShoulder,
            RightUpperArm,
            RightForearm,
            RightHand,
            LeftShoulder,
            LeftUpperArm,
            LeftForearm,
            LeftHand,
            RightUpperLeg,
            RightLowerLeg,
            RightFoot,
            RightToe,
            LeftUpperLeg,
            LeftLowerLeg,
            LeftFoot,
            LeftToe
        }

        [SerializeField] private BoneConverter[] bones;
        [SerializeField] private bool sendData;
        [SerializeField] private int sendFrequencyMs = 500;
        [SerializeField] private string filePath = "/Json/body_positions.json";

        private TcpClient _socket;
        private Thread _writeThread;
        private Queue<string> _dataQueue;

        private void Start()
        {
            if (File.Exists(Application.dataPath + filePath))
            {
                // Clear the file if it exists
                File.WriteAllText(Application.dataPath + filePath, string.Empty);
            }

            _dataQueue = new Queue<string>();
            _writeThread = new Thread(WriteDataToFile);
            _writeThread.Start();
        }

        private void Update()
        {
            string jsonData = ConvertBonePositionsToJson();
            _dataQueue.Enqueue(jsonData);
        }

        private void WriteDataToFile()
        {
            while (true)
            {
                if (sendData && _dataQueue.Count > 0)
                {
                    string jsonData = _dataQueue.Dequeue();
                    // Overwrite existing data with new JSON
                    File.AppendAllText(Application.dataPath + filePath, jsonData + "\n");
                }

                Thread.Sleep(sendFrequencyMs); // Adjust sleep time
            }
        }

        private string ConvertBonePositionsToJson()
        {
            WrapperHolder wrapperHolder = new();
            foreach (BoneConverter bone in bones)
                wrapperHolder.list.Add(new BoneData(bone.xsensBone.ToSpacedString(),
                    ConvertToXsensAxis(bone.boneTransform.position)));
            return JsonUtility.ToJson(wrapperHolder);
        }

        private static Vector3 ConvertToXsensAxis(Vector3 position) => new(position.x, position.z, position.y);

        private void OnDestroy()
        {
            _writeThread.Abort();
        }

        [ContextMenu("Reset Json File And Pause")]
        public void ResetJsonFileAndPause()
        {
            sendData = false;
            File.WriteAllText(Application.dataPath + filePath, string.Empty);
        }

        public static string ToSpacedString(this Enum value)
        {
            return Regex.Replace(value.ToString(), "(?<!^)([A-Z])", " $1");
        }
    }

    [Serializable]
    public class BoneConverter
    {
        public BoneDataSender.XsensBones xsensBone;
        public Transform boneTransform;
    }

    [Serializable]
    public class WrapperHolder
    {
        public List<BoneData> list;

        public WrapperHolder() => list = new List<BoneData>();
        public WrapperHolder(List<BoneData> list) => this.list = list;
    }

    [Serializable]
    public class BoneData
    {
        public string name;
        public Vector3 position;

        public BoneData(string name = "", Vector3 position = default)
        {
            this.name = name;
            this.position = position;
        }
    }
}